"""Code generated by Speakeasy (https://speakeasyapi.dev). DO NOT EDIT."""

from __future__ import annotations
import dataclasses
from billing import utils
from dataclasses_json import Undefined, dataclass_json
from typing import Final, Optional


@dataclass_json(undefined=Undefined.EXCLUDE)

@dataclasses.dataclass
class TrialConfig:
    TRIAL_PERIOD_UNIT: Final[str] = dataclasses.field(default='days', metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('trial_period_unit') }})
    trial_period: Optional[float] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('trial_period') }})
    

