"""Code generated by Speakeasy (https://speakeasyapi.dev). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum

class CreditNoteType(str, Enum):
    r"""Describing if this is an `adjustment` or a `refund`"""
    REFUND = 'refund'
    ADJUSTMENT = 'adjustment'
