"""Code generated by Speakeasy (https://speakeasyapi.dev). DO NOT EDIT."""

from __future__ import annotations
import dataclasses
from billing import utils
from dataclasses_json import Undefined, dataclass_json


@dataclass_json(undefined=Undefined.EXCLUDE)

@dataclasses.dataclass
class PackageConfig:
    r"""Provided when model_type is `package`"""
    package_amount: str = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('package_amount') }})
    package_size: float = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('package_size') }})
    

