"""Code generated by Speakeasy (https://speakeasyapi.dev). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum

class ExternalMarketplace(str, Enum):
    r"""Optionally provide the name of the external marketplace that the subscription is attached to."""
    GOOGLE = 'google'
    AWS = 'aws'
    AZURE = 'azure'
