"""Code generated by Speakeasy (https://speakeasyapi.dev). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum

class Reason(str, Enum):
    r"""A reason for this credit note, which can be one of \\"Duplicate\\", \\"Fraudulent\\", \\"Order change\\", \\"Product unsatisfactory\\" """
    DUPLICATE = 'Duplicate'
    FRAUDULENT = 'Fraudulent'
    ORDER_CHANGE = 'Order change'
    PRODUCT_UNSATISFACTORY = 'Product unsatisfactory'
