"""Code generated by Speakeasy (https://speakeasyapi.dev). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum

class ModelType(str, Enum):
    UNIT = 'unit'
    TIERED = 'tiered'
    BULK = 'bulk'
    PACKAGE = 'package'
    BPS = 'bps'
    TIERED_BPS = 'tiered_bps'
    BULK_BPS = 'bulk_bps'
    MATRIX = 'matrix'
