"""Code generated by Speakeasy (https://speakeasyapi.dev). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum

class CancelOption(str, Enum):
    r"""Determines the timing of subscription cancellation"""
    IMMEDIATE = 'immediate'
    END_OF_SUBSCRIPTION_TERM = 'end_of_subscription_term'
    REQUESTED_DATE = 'requested_date'
