"""Code generated by Speakeasy (https://speakeasyapi.dev). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum

class Action(str, Enum):
    r"""Describes the reason that this transaction took place."""
    APPLIED_TO_INVOICE = 'applied_to_invoice'
    PRORATED_REFUND = 'prorated_refund'
    MANUAL_ADJUSTMENT = 'manual_adjustment'
